@@ -14,10 +14,9 @@ from ipython_genutils.importstring import import_item
 from traitlets import (
     Any, Bool, Dict, DottedObjectName, Instance, Unicode, default, observe
 )
-from ipython_genutils.py3compat import unicode_type
 
 from .kernelspec import NATIVE_KERNEL_NAME, KernelSpecManager
-from .manager import AsyncKernelManager
+from .manager import KernelManager, AsyncKernelManager
 
 
 class DuplicateKernelError(Exception):
@@ -57,7 +56,7 @@ class MultiKernelManager(LoggingConfigurable):
     )
 
     def __init__(self, *args, **kwargs):
-        super(MultiKernelManager, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         # Cache all the currently used ports
         self.currently_used_ports = set()
@@ -76,6 +75,11 @@ class MultiKernelManager(LoggingConfigurable):
         kernel_manager_ctor = import_item(self.kernel_manager_class)
 
         def create_kernel_manager(*args, **kwargs):
+            if self.shared_context:
+                if self.context.closed:
+                    # recreate context if closed
+                    self.context = self._context_default()
+                kwargs.setdefault("context", self.context)
             km = kernel_manager_ctor(*args, **kwargs)
 
             if km.cache_ports:
@@ -104,10 +108,33 @@ class MultiKernelManager(LoggingConfigurable):
 
                 return port
 
+    shared_context = Bool(
+        True,
+        config=True,
+        help="Share a single zmq.Context to talk to all my kernels",
+    )
+
+    _created_context = Bool(False)
+
     context = Instance('zmq.Context')
+
+    @default("context")
     def _context_default(self):
+        self._created_context = True
         return zmq.Context()
 
+    def __del__(self):
+        if self._created_context and self.context and not self.context.closed:
+            if self.log:
+                self.log.debug("Destroying zmq context for %s", self)
+            self.context.destroy()
+        try:
+            super_del = super().__del__
+        except AttributeError:
+            pass
+        else:
+            super_del()
+
     connection_dir = Unicode('')
 
     _kernels = Dict()
@@ -125,8 +152,8 @@ class MultiKernelManager(LoggingConfigurable):
     def __contains__(self, kernel_id):
         return kernel_id in self._kernels
 
-    def pre_start_kernel(self, kernel_name, **kwargs):
-
+    def pre_start_kernel(self, kernel_name, kwargs):
+        # kwargs should be mutable, passing it as a dict argument.
         kernel_id = kwargs.pop('kernel_id', self.new_kernel_id(**kwargs))
         if kernel_id in self:
             raise DuplicateKernelError('Kernel already exists: %s' % kernel_id)
@@ -154,7 +181,7 @@ class MultiKernelManager(LoggingConfigurable):
 
         The kernel ID for the newly started kernel is returned.
         """
-        km, kernel_name, kernel_id = self.pre_start_kernel(kernel_name, **kwargs)
+        km, kernel_name, kernel_id = self.pre_start_kernel(kernel_name, kwargs)
         km.start_kernel(**kwargs)
         self._kernels[kernel_id] = km
         return kernel_id
@@ -201,6 +228,10 @@ class MultiKernelManager(LoggingConfigurable):
     def cleanup(self, kernel_id, connection_file=True):
         """Clean up a kernel's resources"""
 
+    @kernel_method
+    def cleanup_resources(self, kernel_id, restart=False):
+        """Clean up a kernel's resources"""
+
     def remove_kernel(self, kernel_id):
         """remove a kernel from our mapping.
 
@@ -218,7 +249,18 @@ class MultiKernelManager(LoggingConfigurable):
             self.request_shutdown(kid)
         for kid in kids:
             self.finish_shutdown(kid)
-            self.cleanup(kid)
+
+            # Determine which cleanup method to call
+            # See comment in KernelManager.shutdown_kernel().
+            km = self.get_kernel(kid)
+            overrides_cleanup = type(km).cleanup is not KernelManager.cleanup
+            overrides_cleanup_resources = type(km).cleanup_resources is not KernelManager.cleanup_resources
+
+            if overrides_cleanup and not overrides_cleanup_resources:
+                km.cleanup(connection_file=True)
+            else:
+                km.cleanup_resources(restart=False)
+
             self.remove_kernel(kid)
 
     @kernel_method
@@ -399,7 +441,7 @@ class MultiKernelManager(LoggingConfigurable):
         :param kwargs:
         :return: string-ized version 4 uuid
         """
-        return unicode_type(uuid.uuid4())
+        return str(uuid.uuid4())
 
 
 class AsyncMultiKernelManager(MultiKernelManager):
@@ -419,7 +461,7 @@ class AsyncMultiKernelManager(MultiKernelManager):
 
         The kernel ID for the newly started kernel is returned.
         """
-        km, kernel_name, kernel_id = self.pre_start_kernel(kernel_name, **kwargs)
+        km, kernel_name, kernel_id = self.pre_start_kernel(kernel_name, kwargs)
         if not isinstance(km, AsyncKernelManager):
             self.log.warning("Kernel manager class ({km_class}) is not an instance of 'AsyncKernelManager'!".
                              format(km_class=self.kernel_manager_class.__class__))
@@ -508,5 +550,5 @@ class AsyncMultiKernelManager(MultiKernelManager):
             self.request_shutdown(kid)
         for kid in kids:
             await self.finish_shutdown(kid)
-            self.cleanup(kid)
+            self.cleanup_resources(kid)
             self.remove_kernel(kid)
