@@ -5,13 +5,9 @@
 
 import hmac
 import os
-import sys
 import uuid
 from datetime import datetime
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+from unittest import mock
 
 import pytest
 
@@ -23,8 +19,6 @@ from zmq.eventloop.zmqstream import ZMQStream
 from jupyter_client import session as ss
 from jupyter_client import jsonutil
 
-from ipython_genutils.py3compat import string_types
-
 def _bad_packer(obj):
     raise TypeError("I don't work")
 
@@ -58,8 +52,8 @@ class TestSession(SessionTestCase):
         self.assertTrue(isinstance(msg['metadata'],dict))
         self.assertTrue(isinstance(msg['header'],dict))
         self.assertTrue(isinstance(msg['parent_header'],dict))
-        self.assertTrue(isinstance(msg['msg_id'], string_types))
-        self.assertTrue(isinstance(msg['msg_type'], string_types))
+        self.assertTrue(isinstance(msg['msg_id'], str))
+        self.assertTrue(isinstance(msg['msg_type'], str))
         self.assertEqual(msg['header']['msg_type'], 'execute')
         self.assertEqual(msg['msg_type'], 'execute')
 
@@ -141,9 +135,8 @@ class TestSession(SessionTestCase):
 
         # buffers must be contiguous
         buf = memoryview(os.urandom(16))
-        if sys.version_info >= (3,4):
-            with self.assertRaises(ValueError):
-                self.session.send(A, msg, ident=b'foo', buffers=[buf[::2]])
+        with self.assertRaises(ValueError):
+            self.session.send(A, msg, ident=b'foo', buffers=[buf[::2]])
 
         A.close()
         B.close()
@@ -154,17 +147,17 @@ class TestSession(SessionTestCase):
         s = self.session
         self.assertTrue(s.pack is ss.default_packer)
         self.assertTrue(s.unpack is ss.default_unpacker)
-        self.assertEqual(s.username, os.environ.get('USER', u'username'))
+        self.assertEqual(s.username, os.environ.get('USER', 'username'))
 
         s = ss.Session()
-        self.assertEqual(s.username, os.environ.get('USER', u'username'))
+        self.assertEqual(s.username, os.environ.get('USER', 'username'))
 
         self.assertRaises(TypeError, ss.Session, pack='hi')
         self.assertRaises(TypeError, ss.Session, unpack='hi')
         u = str(uuid.uuid4())
-        s = ss.Session(username=u'carrot', session=u)
+        s = ss.Session(username='carrot', session=u)
         self.assertEqual(s.session, u)
-        self.assertEqual(s.username, u'carrot')
+        self.assertEqual(s.username, 'carrot')
 
     def test_tracking(self):
         """test tracking messages"""
@@ -294,8 +287,8 @@ class TestSession(SessionTestCase):
         self.assertEqual(msg['parent_header'], msg2['parent_header'])
         assert isinstance(msg['content']['t'], datetime)
         assert isinstance(msg['metadata']['t'], datetime)
-        assert isinstance(msg2['content']['t'], string_types)
-        assert isinstance(msg2['metadata']['t'], string_types)
+        assert isinstance(msg2['content']['t'], str)
+        assert isinstance(msg2['metadata']['t'], str)
         self.assertEqual(msg['content'], jsonutil.extract_dates(msg2['content']))
         self.assertEqual(msg['content'], jsonutil.extract_dates(msg2['content']))
 
@@ -311,7 +304,7 @@ class TestSession(SessionTestCase):
 
         session = ss.Session(
             pack=msgpack.packb,
-            unpack=lambda buf: msgpack.unpackb(buf, encoding='utf8'),
+            unpack=lambda buf: msgpack.unpackb(buf, raw=False),
         )
         self._datetime_test(session)
 
